package com.zswl.dataservicestarter.domain;

import com.zswl.dataservicestarter.domain.base.DataServiceSuperEntity;
import com.zswl.dataservicestarter.type.DataServiceOrderType;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.DBRef;

import java.math.BigDecimal;
import java.util.List;

import cn.hutool.json.JSONObject;

/**
 * 订单信息
 *
 * @author TRX
 * @date 2024/3/25
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class DataServiceOrderInfo extends DataServiceSuperEntity {

    /**
     * 数据的hashCode
     */
    private Integer hashCode;

    /**
     * 订单类型 平台  商铺
     */
    private DataServiceOrderType orderType;

    /**
     * 是否同步到平台
     */
    private Boolean isSync2Platform;

    /**
     * 是否同步到商铺
     */
    private Boolean isSync2Shop;

    /**
     * 同步时间
     */
    private Long syncTime;

    //----------------------------平台订单信息 start-------------------------

    /**
     * 用户ID
     */
    private String userId;

    /**
     * 店铺ID
     */
    private String epId;

    /**
     * 订单号
     */
    @Indexed(unique = true)
    private String orderNo;

    /**
     * 店铺信息
     */
    private JSONObject shopInfo;

    /**
     * 商品总价格
     */
    private BigDecimal totalAmount;

    /**
     * 优惠价格
     */
    private BigDecimal discountAmount;


    /**
     * 实际支付价格
     */
    private BigDecimal payAmount;

    /**
     * 使用优惠券信息
     */
    private List<JSONObject> discountList;

    /**
     * 退款信息
     */
    @DBRef
    private RefundLog refundLog;

    /**
     * 是否已退款
     */
    private Boolean isRefund;

    /**
     * 扩展参数
     */
    private String extend;

    /**
     * 商品和分账信息
     */
    @DBRef
    private List<DataServiceGoods> goodsList;

    /**
     * 支付信息
     */
    @DBRef
    private DataServicePaymentInfo payment;

    //----------------------------平台订单信息 end---------------------------

}
