package com.zswl.dataservicestarter.utils;

import org.springframework.aop.framework.AdvisedSupport;
import org.springframework.aop.framework.AopProxy;
import org.springframework.aop.support.AopUtils;
import org.springframework.util.ClassUtils;

import java.lang.reflect.Field;

public class AopTargetUtil {

    /**
     * 获取 目标对象
     * @param proxy 代理对象
     * @return 原生对象
     * @throws Exception 异常
     */
    public static Object getTarget(Object proxy) throws Exception {
         if(proxy==null)
         {
             return null;
         }
        if(!AopUtils.isAopProxy(proxy) && !proxy.getClass().getName().contains(ClassUtils.CGLIB_CLASS_SEPARATOR)) {
            return proxy;//不是代理对象
        }

        if(AopUtils.isJdkDynamicProxy(proxy)) {
            return getJdkDynamicProxyTargetObject(proxy);
        } else { //cglib
            return getCglibProxyTargetObject(proxy);
        }

    }


    private static Object getCglibProxyTargetObject(Object proxy) throws Exception {
        Field h = proxy.getClass().getDeclaredField("CGLIB$CALLBACK_0");
        h.setAccessible(true);
        Object dynamicAdvisedInterceptor = h.get(proxy);

        try {
            Field result = dynamicAdvisedInterceptor.getClass().getDeclaredField("result");
            result.setAccessible(true);
            return  result.get(dynamicAdvisedInterceptor);
        }catch (NoSuchFieldException nfe)
        {
            nfe.printStackTrace();
        }
        return  null;
    }


    private static Object getJdkDynamicProxyTargetObject(Object proxy) throws Exception {
        Field h = proxy.getClass().getSuperclass().getDeclaredField("h");
        h.setAccessible(true);
        AopProxy aopProxy = (AopProxy) h.get(proxy);

        Field advised = aopProxy.getClass().getDeclaredField("advised");
        advised.setAccessible(true);

        return ((AdvisedSupport)advised.get(aopProxy)).getTargetSource().getTarget();
    }
}
