package com.zswl.dataservicestarter.utils.result;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.zswl.dataservicestarter.type.ResultState;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.util.Optional;
import java.util.function.Consumer;

/**
 * 结果
 *
 * @param <T>
 */
@Builder
@AllArgsConstructor
@NoArgsConstructor
@ToString(callSuper = true)
@JsonInclude(JsonInclude.Include.NON_NULL)
@ApiModel("API返回封装")
public class ResultContent<T> {

    //内容
    @Getter
    @Setter
    @ApiModelProperty("返回内容")
    private T content;

    //状态
    @Getter
    @Setter
    @ApiModelProperty("状态")
    private ResultState state;

    //文本
    @Getter
    @Setter
    @ApiModelProperty("信息")
    private String msg;


    /**
     * 获取内容
     *
     * @return
     */
    public Optional<T> optionalContent() {
        return Optional.ofNullable(this.content);
    }

    public static <T> ResultContent build(ResultState state, T content) {
        return ResultContent.builder().state(state).content(content).msg(state.getRemark()).build();
    }

    public static <T> ResultContent build(ResultState state, T content, String msg) {
        return ResultContent.builder().state(state).content(content).msg(msg).build();
    }

    public static <T> ResultContent build(ResultState state, String msg) {
        return ResultContent.builder().state(state).msg(msg).build();
    }

    public static <T> ResultContent build(boolean bool) {
        return build(bool ? ResultState.Success : ResultState.Fail, null);
    }

    public static <T> ResultContent buildFail(String msg) {
        return ResultContent.builder().state(ResultState.Fail).msg(msg).build();
    }

    public static <T> ResultContent buildSuccess(String msg) {
        return build(ResultState.Success, msg);
    }

    public static <T> ResultContent buildSuccess(T content) {
        return build(ResultState.Success, content);
    }

    public static <T> ResultContent buildSuccess() {
        return build(ResultState.Success, ResultState.Success.getRemark());
    }

    public static <T> ResultContent build(ResultState state) {
        return build(state, state.getRemark());
    }

    public static <T> ResultContent buildContent(T content) {
        return build(content == null ? ResultState.Fail : ResultState.Success, content);
    }

    public void ifSuccess(Consumer<? super T> action) {
        if (isSuccess() && content != null) {
            action.accept(content);
        }
    }

    public void ifSucessOrElse(Consumer<? super T> action, Runnable emptyAction) {
        if (isSuccess() && content != null) {
            action.accept(content);
        } else {
            emptyAction.run();
        }
    }

    public boolean isSuccess() {
        return ResultState.Success.equals(this.state);
    }

    public boolean isFailed() {
        return !ResultState.Success.equals(this.state);
    }
}
